<?php

namespace Teknisa\ACL\Services;

use Teknisa\ACL\Repositories\AccessProfile as AccessProfileRepository;
use Teknisa\ACL\Repositories\PerfAcessParc;
use Teknisa\ACL\Repositories\ZhuAccessControlRule;
use Teknisa\Libs\Util\Environment;
use Doctrine\ORM\EntityManager;
use Teknisa\ACL\Util\Utilities;
use Teknisa\ACL\Util\InstanceProvider;


class AccessProfile {

    const MAXNRPERFILACESSO = "MAXNRPERFILACESSO";
    const NRORG = "NRORG";
    const NRUSERGROUP = "NRUSERGROUP";
    const NRPERFACESSOPAI = "NRPERFACESSOPAI";
    const NRORGINCLUSAO = "NRORGINCLUSAO";
    const NRORGULTATU = "NRORGULTATU";
    const DTINCLUSAO = "DTINCLUSAO";
    const DTULTATU = "DTULTATU";
    const CDOPERINCLUSAO = "CDOPERINCLUSAO";
    const CDOPERULTATU = "CDOPERULTATU";
    const NRPERFILACESSO = "NRPERFILACESSO";

    /** @var AccessProfileRepository */
    protected $accessProfileRepository;

    /** @var PerfAcessParc */
    protected $perfAcessParcRepository;

    /** @var ZhuAccessControlRule */
    protected $accessControlRuleRepository;

    /** @var Environment */
    protected $environment;

    public function __construct(AccessProfileRepository $accessProfileRepository, PerfAcessParc $perfAcessParcRepository,
                                ZhuAccessControlRule $accessControlRuleRepository, Environment $environment) {
        $this->accessProfileRepository = $accessProfileRepository;
        $this->perfAcessParcRepository = $perfAcessParcRepository;
        $this->accessControlRuleRepository = $accessControlRuleRepository;
        $this->environment = $environment;
    }

    public function getMaxNrPerfilAcesso() {
        return InstanceProvider::getNewCode()->getCodeByTableName('PERFILACESSO', $this->environment->getCurrentOrganizationId());
    }

    public function setDefaultValues(&$row) {
        $this->setNrOrgValues($row);
        $this->setDateValues($row);
        $this->setOperatorValues($row);

        $row[self::NRUSERGROUP] = null;
        $row[self::NRPERFACESSOPAI] = null;
    }

    protected function setNrOrgValues(&$row) {
        $nrOrg = $this->getNrOrgFilter();
        $row[self::NRORG] = $this->setValueIfDontExist($row, self::NRORG, $nrOrg);
        $row[self::NRORGINCLUSAO] = $this->setValueIfDontExist($row, self::NRORGINCLUSAO, $nrOrg);
        $row[self::NRORGULTATU] = $nrOrg;
    }

    protected function setDateValues(&$row) {
        $time = new \DateTime();
        $timeFormatted = $time->format('d/m/Y');
        $row[self::DTINCLUSAO] = $this->setValueIfDontExist($row, self::DTINCLUSAO, $timeFormatted);
        $row[self::DTULTATU] = $timeFormatted;
    }

    protected function setOperatorValues(&$row) {
        $cdOperador = $this->environment->getCurrentUserId();
        $row[self::CDOPERINCLUSAO] = $this->setValueIfDontExist($row, self::CDOPERINCLUSAO, $cdOperador);
        $row[self::CDOPERULTATU] = $cdOperador;
    }

    public function deleteDependencies($profilesToDelete) {
        $params = array (
            self::NRORG =>  $this->getNrOrgFilter()
        );
        $this->accessControlRuleRepository->deleteAssociatedRules($profilesToDelete, $params);
        $this->perfAcessParcRepository->deleteAssociatedPerfAcessParc($profilesToDelete, $params);
        $this->accessProfileRepository->deleteAssociatedPerfAcessFili($profilesToDelete, $params);
    }

    public function getNrOrgFilter() {
        return $this->environment->getCurrentOrganizationId();
    }

    protected function setValueIfDontExist($row, $property, $value) {
        return isset($row[$property]) ? $row[$property] : $value;
    }
    
    public function findAccessControllUtilization(){
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        return $this->accessProfileRepository->findAccessControllUtilization($nrorg);
    }

    public function validateActivation(){
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        return $this->accessProfileRepository->validateActivation($nrorg);
    }
    
    public function changeAccessProfile($idutctrlacesweb){
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        $findAccessControllUtilization = $this->findAccessControllUtilization($nrorg);
        $idaces = $idutctrlacesweb == 'N'?'S':'N';

        if(!empty($findAccessControllUtilization)) {
            $this->accessProfileRepository->changeAccessProfile($nrorg, $idaces);
        } else {
            $interval = 1;
            $colSize = 4;
            $cdpargeral = InstanceProvider::getNewCode()->getCodeByTableName('PARAMGERAL', $nrorg, $interval, $colSize);
            $this->accessProfileRepository->insertAccessProfile($nrorg, $idaces, $cdpargeral);
        }

        return array('idutctrlacesweb' => $idaces);
    }
    
    public function findPerfAcessControl($nrAccessProfile){
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        return $this->accessProfileRepository->findPerfAcessControl($nrorg, $nrAccessProfile);
    }
    
    public function findPerfAccessByControlProfile($nrAccessProfile){
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        return $this->accessProfileRepository->findPerfAccessByControlProfile($nrorg, $nrAccessProfile);
    }
    
    public function savePerfAccessByControlProfile($rows){
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $libEnvironment = InstanceProvider::getLibEnvironment();
            $organizationId = $libEnvironment->getCurrentOrganizationId();
            foreach($rows as $accessProfile){
                $accessProfile['NRORG']             = $organizationId;
                $accessProfile['NRORGINCLUSAO']     = $organizationId;
                $accessProfile['CDOPERINCLUSAO']    = $libEnvironment->getCurrentUserId();
                $this->accessProfileRepository->saveAccessProfileByControlProfile($accessProfile);
            }
            $entityManager->flush();
            $connection->commit();
            return (array());
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deleteControlProfile($nrAccesProfile, $nrControlProfile)
    {
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
         $this->accessProfileRepository->deleteControlProfile($nrAccesProfile, $nrControlProfile, $organizationId);
    }
    
    public function changeAccessControl($idutlperfilfilial){
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        $this->accessProfileRepository->changeAccessControl($nrorg, $idutlperfilfilial);
        return array('idutlperfilfilial' => $idutlperfilfilial);
    }
}